
/*
 * Copyright 2010 - 2020 Anywhere Software (www.b4x.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package anywheresoftware.b4a.objects;

import com.amazon.device.messaging.ADMMessageHandlerBase;
import com.amazon.device.messaging.ADMMessageReceiver;

import android.app.Service;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.Handler;

import anywheresoftware.b4a.AbsObjectWrapper;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.Events;
import anywheresoftware.b4a.BA.Hide;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.B4AClass;
import anywheresoftware.b4a.keywords.Common;
//Resources used:
//https://developer.amazon.com/docs/adm/integrate-your-app.html
//https://www.programcreek.com/java-api-examples/?code=daffodilistic%2Faws-snsmobilepush%2Faws-snsmobilepush-master%2FKindleMobilePushApp%2Fsrc%2Fcom%2Famazonaws%2Fkindletest%2FADMMessageHandler.java#
//https://github.com/AnywhereSoftware/B4A/blob/master/Libs_FirebaseNotifications/src/anywheresoftware/b4a/objects/FirebaseNotificationsService.java

@Hide
public class AmazonNotificationsService extends ADMMessageHandlerBase {
	private static boolean firstMessage = true;
	private static Handler handler;
	private static Class<?> ServiceClass;
	private static Class<?> ReceiverClass;
	
	protected AmazonNotificationsService(String className) {
		super(className);
	}
	
	public AmazonNotificationsService() {
		super(AmazonNotificationsService.class.getName());
	}

	// Receiver portion
	@Hide
	public static class AmazonMessageReceiver extends ADMMessageReceiver {
		public AmazonMessageReceiver() {
			super(AmazonNotificationsService.class);
			BA.Log("AmazonMessageReceiver instantiation!");
		}
	}
	
	@Override
	public void onCreate() {
		super.onCreate();
		BA.Log("AmazonNotificationsService onCreate()");
		try {
			handler = new Handler();
			//Note: Service module in B4A must be named AmazonMessaging (caps not important)
			ServiceClass = Class.forName(getPackageName() + ".amazonmessaging");
			ReceiverClass = Class.forName(getPackageName() + ".amazonmessaging$amazonmessaging_BR");
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
			BA.LogError("AmazonMessaging not found.");
		}
	}

	public static Intent createIntent(Service me, String event) {
		Intent i;
		i = new Intent(me, ReceiverClass);
		i.setAction("b4a_amazonmessaging");
		i.putExtra("event", event);
		return i;
	}
	
	@Override
	protected void onMessage(final Intent intent) {
		BA.Log("AmazonNotificationsService.OnMessage("+intent+")");
		handler.post(new Runnable() {
			@Override
			public void run() {
				final Intent i = createIntent(AmazonNotificationsService.this, "b4a_onmessage");
				i.putExtra("message", intent);
				try {
					if (firstMessage || Common.IsPaused(null, ServiceClass) == false) {
						firstMessage = false;
							sendBroadcast(i);
					} else {
						BA.handler.postDelayed(new Runnable() {

							@Override
							public void run() {
								sendBroadcast(i);
							}

						}, 500);
					}
				} catch (Exception e) {
					throw new RuntimeException(e);
				}
			}});
	}


	@Override
	protected void onRegistered(String arg0) {
		BA.Log("AmazonNotificationsService.OnRegistered("+arg0+")");
		// TODO Auto-generated method stub
		final Intent i = createIntent(AmazonNotificationsService.this, "b4a_onregistered");
		i.putExtra("string", arg0);
		BA.applicationContext.sendBroadcast(i);
	}

	@Override
	protected void onRegistrationError(String arg0) {
		// TODO Auto-generated method stub
		BA.Log("AmazonNotificationsService.OnRegistrationError("+arg0+")");
		final Intent i = createIntent(AmazonNotificationsService.this, "b4a_onerror");
		i.putExtra("string", arg0);
		BA.applicationContext.sendBroadcast(i);
	}

	@Override
	protected void onUnregistered(String arg0) {
		// TODO Auto-generated method stub
		final Intent i = createIntent(AmazonNotificationsService.this, "b4a_onunregistered");
		i.putExtra("string", arg0);
		BA.applicationContext.sendBroadcast(i);
	}

	// @Hide
	// public static class InstanceService extends FirebaseInstanceIdService {
	// @Override
	// public void onTokenRefresh() {
	// sendBroadcast(createIntent(this, "b4a_tokenrefresh"));
	// }
	// }
	// @DependsOn(values={"com.google.firebase:firebase-messaging",
	// "com.google.firebase:firebase-installations"})
	@ShortName("AmazonMessaging")
	@Events(values={"Error (Info As String)", "Registered (Info As String)", "Unregistered (Info As String)", "MessageArrived (Message As String)"})
	public static class AmazonMessageWrapper {
		private BA ba;
		private String eventName;

		public void Initialize(BA ba, String EventName) {
			this.ba = ba;
			this.eventName = EventName.toLowerCase(BA.cul);
		}
	
			/**
		 * Should be called from Service_Start. Returns true if the intent was handled.
		 */
		public boolean HandleIntent(Intent Intent) {
			if (Intent == null || "b4a_amazonmessaging".equals(Intent.getAction()) == false) {
				BA.Log("AmazonMessageWrapper HandleIntent: Not an Intent for us!");
				return false;
			}
			Intent.setExtrasClassLoader(Intent.class.getClassLoader());
			String event = Intent.getStringExtra("event");
			if (event.equals("b4a_onerror"))
				ba.raiseEventFromUI(this, eventName + "_error", getData(Intent));
			else if (event.equals("b4a_onregistered"))
				ba.raiseEventFromUI(this, eventName + "_registered", getData(Intent));
			else if (event.equals("b4a_unregistered"))
				ba.raiseEventFromUI(this, eventName + "_unregistered", getData(Intent));
			else if (event.equals("b4a_onmessge")) {
				ba.raiseEventFromUI(this, eventName + "_messagearrived", ExtractMessage(Intent));
				ExtractMessage(Intent);
			}
			return true;
		}
		
		private String getData(Intent intent) {
			return(intent.getStringExtra("string"));
		}

		private String ExtractMessage(Intent intent) {
			//Note: Using string constants directly here. Should be changed
			final String msgKey = "default"; //getString(R.string.json_data_msg_key);
			//final String timeKey = "timeStamp"; //getString(R.string.json_data_time_key);
			
			final Intent i2 = intent.getParcelableExtra("message");
			return(i2.getStringExtra(msgKey));
		}
		
	}
	
}
